{
#######################################################################################
#  // The MIT License (MIT):                                                          #
#                                                                                     #
#  Synapse Example - Server with multiple Clients                                     #
#  Copyright (c) 2014   Marcel Märtens                                                #
#                                                                                     #
#  Permission is hereby granted, free of charge, to any person obtaining a copy       #
#  of this software and associated documentation files (the "Software"), to deal      #
#  in the Software without restriction, including without limitation the rights       #
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell          #
#  copies of the Software, and to permit persons to whom the Software is              #
#  furnished to do so, subject to the following conditions:                           #
#                                                                                     #
#  The above copyright notice and this permission notice shall be included in         #
#  all copies or substantial portions of the Software.                                #
#                                                                                     #
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR         #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,           #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE        #
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER             #
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,      #
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN          #
#  THE SOFTWARE.                                                                      #
#######################################################################################
}

// Version 1.1 - 08.12.2014 - posted on http://www.lazarusforum.de/

unit synapseExample_1;

{$mode objfpc}{$H+}

interface

uses
 Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
 ComCtrls, synapseexample_server, synapseexample_client;

type

 { TClientWithHistory }

 TClientWithHistory = class(TClient)
   History:TStrings;
   procedure IReceivedSomething(aMSG:String); override;
   constructor Create(CreateSuspended: Boolean;const StackSize: SizeUInt = DefaultStackSize);
 end;

 { TForm1 }

 TForm1 = class(TForm)
  Button1: TButton;
  Button2: TButton;
  Button3: TButton;
  Edit1: TEdit;
  Edit2: TEdit;
  GroupBox1: TGroupBox;
  GroupBox2: TGroupBox;
  Memo1: TMemo;
  Memo2: TMemo;
  TabControl1: TTabControl;
  procedure Button1Click(Sender: TObject);
  procedure Button2Click(Sender: TObject);
  procedure Button3Click(Sender: TObject);
  procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
  procedure FormCreate(Sender: TObject);
  procedure TabControl1Change(Sender: TObject);
 private
  { private declarations }
 public
  { public declarations }
  procedure OnRecieve(aServer:TServer;aClient:TServerClient;aMSG:String);
 public
  Server:TServer;
  Client:array of TClientWithHistory;
  function CurClient:TClientWithHistory;
  procedure UpdateGUI;
 end;

var
 Form1: TForm1;

implementation

{$R *.lfm}

{ TClientWithHistory }

procedure TClientWithHistory.IReceivedSomething(aMSG: String);
begin
 inherited IReceivedSomething(aMSG);
 History.Add(aMSG);
 TThread.Synchronize(nil,@Form1.UpdateGUI);
end;

constructor TClientWithHistory.Create(CreateSuspended: Boolean;
 const StackSize: SizeUInt);
begin
 inherited;
 History := TStringList.Create;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  Server:=TServer.Create;
  Server.onMessage:= @OnRecieve;
  Server.Start(7777);
end;

procedure TForm1.TabControl1Change(Sender: TObject);
begin
Memo2.Lines := CurClient.History;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
Server.BroadCast(Edit1.Text);
Edit1.Text:= '';
Edit1.SetFocus;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
CurClient.Send(Edit2.Text);
Edit2.Text:= '';
Edit2.SetFocus;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
setlength(Client,length(Client)+1);
Client[high(Client)] := TClientWithHistory.Create(True);
Client[high(Client)].Start('127.0.0.1',7777);
Client[high(Client)].Send('New Client ' + InttoStr(length(Client)) +  ' Sending the first Message');
TabControl1.Tabs.Add('Client ' + InttoStr(length(Client)));
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
Server.Stop;
end;

procedure TForm1.OnRecieve(aServer: TServer; aClient: TServerClient;
 aMSG: String);
var
 i:integer;
begin
for i := 0 to high(aServer.Clients) do
    begin
    if aServer.Clients[i] = aClient then Memo1.Lines.Add('Client ' + InttoStr(i+1) + ':  ' + aMSG);
    end;
end;

function TForm1.CurClient: TClientWithHistory;
begin
Result := Client[TabControl1.TabIndex];
end;

procedure TForm1.UpdateGUI;
begin
TabControl1Change(nil);
end;

end.

