{
#######################################################################################
#  // The MIT License (MIT):                                                          #
#                                                                                     #
#  Synapse Example - Server with multiple Clients                                     #
#  Copyright (c) 2014   Marcel Märtens                                                #
#                                                                                     #
#  Permission is hereby granted, free of charge, to any person obtaining a copy       #
#  of this software and associated documentation files (the "Software"), to deal      #
#  in the Software without restriction, including without limitation the rights       #
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell          #
#  copies of the Software, and to permit persons to whom the Software is              #
#  furnished to do so, subject to the following conditions:                           #
#                                                                                     #
#  The above copyright notice and this permission notice shall be included in         #
#  all copies or substantial portions of the Software.                                #
#                                                                                     #
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR         #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,           #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE        #
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER             #
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,      #
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN          #
#  THE SOFTWARE.                                                                      #
#######################################################################################
}

unit synapseexample_server;

{$mode objfpc}{$H+}

interface

uses
 Classes, SysUtils,synsock,blcksock,synapseexample_client;

type
  TServer = class;

  { TServerClient }

  {
    a Server Client is a Client which  belongs to a Server.
    It just notifies the Server that Client x has Send a Message,
    The Server needs 1 TServerClient for every TCP Connection,
    which means the Server creates 1 Thread for every TCP Connection
  }
  TServerClient = class(TClient)
    master:TServer;
  protected
    procedure IReceivedSomething(aMSG:String); override;
  end;

  { TServerAcceptThread }

  {
   This is a Special Thread.
   It has his Own connection, and This is the Connection which actually listens on the Port.
   As soon as it accepts a new connection, it Creates a TServerClient, give him its Sock and
   continue listening for the next Client connecting
  }
  TServerAcceptThread = class(TThread)
  public
    master:TServer;
    Socket: TTCPBlockSocket;
  protected
   procedure Execute; override;
  end;

  TOnMessageProc = procedure(aServer:TServer;aClient:TServerClient;aMSG:String) of object;

  {
   Simple Server,
   has multiple Clients and 1 AcceptThread.
   It has a Function to realise a Broadcast
   Set onMessage for a Event
  }
  TServer = class
  private
   procedure InternRecvMessage(from:TServerClient;Message:String);
  public
   Clients:array of TServerClient;
   AcceptThread:TServerAcceptThread;
   onMessage :TOnMessageProc;
   constructor Create;
   procedure Start(Port:Integer);
   procedure Stop;
   procedure BroadCast(aMSG:String);
  end;

implementation

{ TServer }

procedure TServer.InternRecvMessage(from: TServerClient; Message: String);
begin
Message := Message;
if assigned(onMessage) then onMessage(Self,From,Message);
end;

constructor TServer.Create;
begin
inherited;
AcceptThread := TServerAcceptThread.Create(True);
AcceptThread.master := Self;
end;

procedure TServer.Start(Port: Integer);
begin
// This Socket will listen for new Connections on Port X
AcceptThread.Socket := TTCPBlockSocket.Create;
AcceptThread.Socket.Bind(cAnyHost,InttoStr(Port));
AcceptThread.Socket.Listen;
AcceptThread.Resume;
end;

procedure TServer.Stop;
begin
AcceptThread.Terminate;
end;

procedure TServer.BroadCast(aMSG: String);
var
  i:integer;
begin
for i := 0 to high(Clients) do
    begin
    Clients[i].Send(aMSG);
    end;
end;

{ TServerAcceptThread }

procedure TServerAcceptThread.Execute;
var
  newconnection:TSocket;
  newclient:TServerClient;
begin
while not Terminated do
      begin
      newconnection := Socket.Accept;
      if Self.Terminated then exit;
      setlength(master.Clients,length(master.Clients)+1);
      newclient := TServerClient.Create(True);
      newclient.master := master;
      newclient.Socket := TTCPBlockSocket.Create; // new connection for every Client
      newclient.Socket.Socket:= newconnection; // This should use "my handle" so that TServerAcceptThread can continue listening
      newclient.Resume;

      master.Clients[high(master.Clients)] := newclient;
      end;
end;

{ TServerClient }

procedure TServerClient.IReceivedSomething(aMSG: String);
begin
Master.InternRecvMessage(Self,aMSG);
end;

end.

